/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

package GUI;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/*********************************************************
* Class for catching events when user pushes check boxes
*
* @author Olav Haugan
*
* Date: 14.10.01
*********************************************************/
public class CheckBoxAction implements ActionListener
{
	private MQCFrame parent;
	private ButtonGroup buttons;
	private Vector chkBoxButtons;
	private JTextField textField;
	private JComboBox comboBox;
	
	/*********************************************************
	* Constructor. Initializes variables
	*
	* @param MQCFrame The frame that contains menu and buttons
	*
	* @param ButtonGroup The buttons we want to control
	*********************************************************/
	public CheckBoxAction( MQCFrame parent, ButtonGroup buttons )
	{
		this.parent = parent;
		this.buttons = buttons;
		this.textField = null;
		this.chkBoxButtons = null;
		this.comboBox = null;				
	}

	/*********************************************************
	* Constructor. Initializes variables
	*
	* @param MQCFrame The frame that contains menu and buttons
	*
	* @param Vector A vector of check boxes
	*********************************************************/
	public CheckBoxAction( MQCFrame parent, Vector chkBoxButtons )
	{
		this.parent = parent;
		this.chkBoxButtons = chkBoxButtons;
		this.buttons = null;
		this.textField = null;
		this.comboBox = null;		
	}

	/*********************************************************
	* Constructor. Initializes variables
	*
	* @param MQCFrame The frame that contains menu and buttons
	*
	* @param JComboBox The combobox we want to control
	*********************************************************/
	public CheckBoxAction( MQCFrame parent, JComboBox comboBox )
	{
		this.parent = parent;
		this.comboBox = comboBox;
		this.buttons = null;
		this.textField = null;
		this.chkBoxButtons = null;
	}

	/*********************************************************
	* Constructor. Initializes variables
	*
	* @param MQCFrame The frame that contains menu and buttons
	*
	* @param JTextField The text field to enable or disable
	*********************************************************/
	public CheckBoxAction( MQCFrame parent, JTextField textField )
	{
		this.parent = parent;
		this.buttons = null;
		this.chkBoxButtons = chkBoxButtons;
		this.textField = textField;
		this.comboBox = null;				
	}

	/*********************************************************
	* Function is called by the underlying framework 
	* when a check box is clicked.
	*
	* @param ActionEvent The event that happens when a checkbox
	*                    is clicked.
	*********************************************************/
	public void actionPerformed( ActionEvent ev )
	{
		JToggleButton source = (JToggleButton) ev.getSource();
		boolean enabled = source.isSelected();
		
		Enumeration e = null;
		if(buttons == null && chkBoxButtons != null)
		{
			e = chkBoxButtons.elements();
		}
		else if(buttons != null)
		{
			e = buttons.getElements();
		}
		else if(comboBox != null)
		{
			comboBox.setEnabled(enabled);
			return;
		}
		
		setEnabled(e, enabled);
		if(textField != null)
		{
			textField.setEditable(enabled);
		}
		parent.setCheckProfileIcon(false);
	}
	
	/*********************************************************
	* Sets the enumeration of buttons to enabled or disable
	*
	* @param Enumeration The enumeration of buttons to enable
	*                    or disable.
	*
	* @param boolean true to enable button, false to disable
	*
	*********************************************************/
	private void setEnabled(Enumeration e, boolean enabled)
	{
		if(e !=null)
		{			
			while(e.hasMoreElements())
			{
				((JToggleButton) e.nextElement()).setEnabled(enabled);
			}
		}
	}

}
