/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

// $Id: CodeGenX.java,v 1.3 2003/09/08 18:53:02 rjohnsto Exp $

package ZEN_IDL.be;

import java.io.*;
import java.util.*;
import ZEN_IDL.ast.*;
import ZEN_IDL.be.helperUtils.*;

public class CodeGenX extends CodeGenAbstract
{

	public static String getFullName( Node snode , String seperator )
	{
		Token start = snode.getFirstToken();
		Token end = snode.getLastToken();

		String ret = start.toString();
		while( start != end )
		{
			start = start.next;
			ret += seperator;
			ret += start.toString();
		}
		return ret;
	}

	public static boolean isPrimitiveType( String IDLType ){
                IDLType = IDLType.trim();
                Hashtable conv = new Hashtable();

                conv.put( "octet" , "byte" );
                conv.put( "boolean", "boolean" );
                conv.put( "char" , "char" );
                conv.put( "wchar", "char" );
                conv.put( "string", "java.lang.String" );
                conv.put( "wstring", "java.lang.String" );
                conv.put( "short", "short" );
                conv.put( "unsigned short", "short" );
                conv.put( "long", "int" );
                conv.put( "unsigned long", "int" );
                conv.put( "long long", "long" );
                conv.put( "unsigned long long", "long" );
                conv.put( "float", "float" );
                conv.put( "double", "double" );
                conv.put( "void", "void" );
		conv.put( "any", "any" );

                try{
                        conv.get( IDLType ).toString();
                        return true;
                }
                catch( Exception e )
                {
                        return false;
                }
        }

	public static String mapReturnType( String IDLType ){
		String retVal = mapVariableType( IDLType );
		if( retVal.trim().equals( "" ) )
			return "void";
		return retVal;
	}

        public static String mapVariableType( String IDLType )
        {
                IDLType = IDLType.trim();
                Hashtable conv = new Hashtable();

                conv.put( "octet" , "byte" );
                conv.put( "boolean", "boolean" );
                conv.put( "char" , "char" );
                conv.put( "wchar", "char" );
                conv.put( "string", "String" );
                conv.put( "wstring", "String" );
                conv.put( "short", "short" );
                conv.put( "unsigned short", "short" );
                conv.put( "long", "int" );
                conv.put( "unsigned long", "int" );
                conv.put( "long long", "long" );
                conv.put( "unsigned long long", "long" );
                conv.put( "float", "float" );
                conv.put( "double", "double" );
                conv.put( "void", "" );
		conv.put( "any", "org.omg.CORBA.Any" );

                String ret = "";
                try{
                        ret = conv.get( IDLType ).toString();
			System.out.println( "Converting IDLType: " + IDLType + " to " + ret );
                }
                catch( Exception e )
                {
                        ret = IDLType;          //is not one of the primitives then it is an object
                }
                return ret;
        }

        public static String mapVariableHolder( String IDLType )
        {
                IDLType = IDLType.trim();
                Hashtable conv = new Hashtable();

                conv.put( "octet" , "org.omg.CORBA.ByteHolder" );
                conv.put( "boolean", "org.omg.CORBA.BooleanHolder" );
                conv.put( "char" , "org.omg.CORBA.CharHolder" );
                conv.put( "wchar", "org.omg.CORBA.CharHolder" );
                conv.put( "string", "java.lang.String" );
                conv.put( "wstring", "java.lang.String" );
                conv.put( "short", "org.omg.CORBA.ShortHolder" );
                conv.put( "unsigned short", "org.omg.CORBA.ShortHolder" );
                conv.put( "long", "org.omg.CORBA.IntHolder" );
                conv.put( "unsigned long", "org.omg.CORBA.IntHolder" );
                conv.put( "long long", "org.omg.CORBA.LongHolder" );
                conv.put( "unsigned long long", "org.omg.CORBA.LongHolder" );
                conv.put( "float", "org.omg.CORBA.FloatHolder" );
                conv.put( "double", "org.omg.CORBA.DoubleHolder" );

                String ret="";
                try{
                        ret = conv.get( IDLType ).toString();
                }
                catch( Exception e )
                {
                        ret = IDLType + "Holder";
                }
                return ret;
        }

        public static void generateConstantCode( String nPackage , String path , boolean inInterface , String IDLType , String name , String value , PrintWriter out )
        {
                String javaType = mapVariableType( IDLType );
                if( inInterface )
                {
//                      System.out.println( "DEBUG: " + out );
//                      System.out.println( "DEBUG: " +  javaType);
//                      System.out.println( "DEBUG: " +  name);
//                      System.out.println( "DEBUG: " +  value);
                        out.println( "  " + javaType  + " " + name + " = (" + javaType + ") (" + value + ");");
                }else{
                        PrintWriter pw = null;
                        try{
                                pw = new PrintWriter(new BufferedWriter(new FileWriter( path.trim() + name.trim() + ".java" )));
                        }catch( IOException ioex ){
                                System.out.println( "Error creating file: " + name.trim() + ".java" );
                        }

                        pw.println(  nPackage.trim() );
                        pw.println( "public interface " + name.trim() + "{" );
                        pw.println( "   " + javaType  + " value = (" + javaType + ") (" + value + ");");
                        pw.println( "}" );
                        pw.close();
                }
        }

        public static void generateEnumCode( String nPackage , String path , String name , Vector elements )
        {
                java.io.PrintWriter pw = null;
                try{
                        pw = new java.io.PrintWriter(new BufferedWriter(new FileWriter( path.trim() + name.trim() + ".java" )));
                }catch( IOException ioex ){
                        System.out.println( "Error creating file: " + name.trim() + ".java" );
                }

                pw.println( nPackage.trim() );
                pw.println(  );
                pw.println( "public class " + name );
                pw.println( "           implements org.omg.CORBA.portable.IDLEntity {" );
                pw.println( );
                for( int i = 0 ; i < elements.size() ; i++ )
                {
                        String elementName = elements.elementAt(i).toString();
                        pw.println( "   public static final int _" + elementName + " = " + i + ";" );
                        pw.println( "   public static final " + name + " " + elementName + " = new " + name + "(_" + elementName + ");" );
                        pw.println( );
                }
                pw.println( "   int val;" );
                pw.println( "   public int value() { return val; }");
                pw.println( "   public static " + name + " from_int( int value )");
		pw.println( "   {" );
		pw.println( "           "+name+" result = null;");
                pw.println( "           switch( value )");
		pw.println( "           {" );

                for( int j=0 ; j < elements.size() ; j++ )
                {
                        String elementName = elements.elementAt(j).toString();
                        pw.println( "           case " + j + ":");
                        pw.println( "                   result = " + elementName + ";" );
                        pw.println( "                   break;");
                }
                pw.println( "           }");
		pw.println( "           return result;");
                pw.println( "   }");
                pw.println( "   protected " + name + "( int value ){ val = value; }" );
                pw.println( "}");
                pw.close();
        }

        public static void generateLiteralCode( String literalValue , PrintWriter pw )
        {
                pw.print( " " + literalValue + " "  );
                pw.flush();
        }

        public static void genetareScopedNameCode( Vector scopedNameElements , boolean absoluteScope , PrintWriter pw )
        {
                //what do we do with absolute scoped names ?

                for( int i = 0 ; i < scopedNameElements.size()-1 ; i++ )
                        pw.print( scopedNameElements.elementAt( i ).toString() + "." );
                pw.print( scopedNameElements.elementAt( scopedNameElements.size()-1 ).toString() );     //there has to be atleast 1 element
        }

        /*-------------------------------------- INTERFACE CODE ---------------------------------------------
                *interface generates 2 classes. <interfaceName> and <interfaceName>Operations.
                *interfaceOperations contain the methods and the other class is normally empty.
                *abstract interfaces are not currently supported
        ---------------------------------------------------------------------------------------------------*/
        public static void generateInterface( PrintWriter pw , String interfaceName , InterfaceInheritence iSpec , Vector interfaceMethods , VisitorState mState )
        {
		Vector inheritenceSpec = iSpec.getInheritence();
                pw.print( "public interface " + interfaceName + " extends " + interfaceName + "Operations " );
                for( int i = 0 ; i < inheritenceSpec.size() ; i++ )
                {
                        String inheritClassName = InterfaceInheritence.getInterfaceName( (Vector)
				inheritenceSpec.elementAt(i));
			String inheritPackageName = InterfaceInheritence.getPackage( (Vector)
				inheritenceSpec.elementAt(i));

                        if( !mState.isInterfaceAbstract( inheritClassName , inheritPackageName ) )
                                pw.print( ", " + inheritClassName );
                }
                pw.print( ", org.omg.CORBA.portable.IDLEntity , org.omg.CORBA.Object" );
                pw.println( "{" );
                pw.println( "}" );
                pw.flush();
        }

        public static void generateInterfaceOperationsStart( PrintWriter pw , String interfaceName , InterfaceInheritence iSpec , VisitorState mState )
        {
		Vector inheritenceSpec = iSpec.getInheritence();
                pw.print( "public interface " + interfaceName + "Operations " );
                if( inheritenceSpec.size() > 0 )
                {
                        pw.print( " extends " );
                        for( int i = 0 ; i < inheritenceSpec.size() ; i++ )
                        {
				String inheritClassName = InterfaceInheritence.getInterfaceName(
					(Vector)inheritenceSpec.elementAt(i));
				String inheritPackageName = InterfaceInheritence.getPackage( 
					(Vector)inheritenceSpec.elementAt(i));

                                if( i != 0 )
                                        pw.print( " , " );
                                if( mState.isInterfaceAbstract( inheritClassName,inheritPackageName ) )
                                {

                                        pw.print( inheritClassName );
                                }
                                else
                                {
                                        pw.print( inheritClassName + "Operations" );
                                }
                        }
                }
                pw.println( "{" );
                pw.flush();
        }

        public static void generateAbstractInterfaceOperationsStart( PrintWriter pw , String interfaceName , InterfaceInheritence iSpec , VisitorState mState )
        {
		Vector inheritenceSpec = iSpec.getInheritence();
                pw.print( "public interface " + interfaceName );
                pw.print( " extends org.omg.CORBA.portable.IDLEntity " );
                for( int i = 0 ; i < inheritenceSpec.size() ; i++ )
                {
			String inheritClassName = InterfaceInheritence.getInterfaceName(
				(Vector)inheritenceSpec.elementAt(i));
			String inheritPackageName = InterfaceInheritence.getPackage( 
				(Vector)inheritenceSpec.elementAt(i));

                        pw.print( " , " );
                        if( mState.isInterfaceAbstract( inheritClassName , inheritPackageName ) )
                        {
                                pw.print( inheritClassName );
                        }else{
                                pw.print( inheritClassName + "Operations" );
                        }
                }
                pw.println( "{" );
                pw.flush();
        }

        public static void generateClosingBrace( PrintWriter pw )
        {
                pw.println( "}" );
                pw.flush();
        }
        //---------------------------------------------------------------------------------------------------

        public static void generateLeftParen( PrintWriter pw )
        {
                pw.print( " ( " );
                pw.flush();
        }
        public static void generateRightParen( PrintWriter pw )
        {
                pw.print( " ) " );
                pw.flush();
        }
        public static void generatePlus( PrintWriter pw )
        {
                pw.print( "+" );
                pw.flush();
        }
        public static void generateMinus( PrintWriter pw )
        {
                pw.print( "-" );
                pw.flush();
        }
        public static void generateMultiply( PrintWriter pw )
        {
                pw.print( "*" );
                pw.flush();
        }
        public  static void generateDivide( PrintWriter pw )
        {
                pw.print( "/" );
                pw.flush();
        }
        public static void generateOr( PrintWriter pw )
        {
                pw.print( "||" );
                pw.flush();
        }
        public static void generateAnd( PrintWriter pw )
        {
                pw.print( "&&" );
                pw.flush();
        }
        public static void generateXOr( PrintWriter pw )
        {
                pw.print( "^" );
                pw.flush();
        }
        public static void generateTilde( PrintWriter pw )
        {
                pw.print( "~" );
                pw.flush();
        }
        public static void generateLeftShift( PrintWriter pw )
        {
                pw.print( "<<" );
                pw.flush();
        }
        public static void generateRightShift( PrintWriter pw )
        {
                pw.print( ">>" );
                pw.flush();
        }
        public static void generatePercent( PrintWriter pw )
        {
                pw.print( "%" );
                pw.flush();
        }
	public static PrintWriter makePrintWriter( String packagePath, String fileName )
	        throws java.io.IOException
        {
	    return new PrintWriter(new BufferedWriter(new FileWriter( packagePath.trim() + fileName.trim() + ".java" )));
        }
        public static PrintWriter makePrintWriter( String packageString , String packagePath , String fileName )
                throws java.io.IOException
        {
                PrintWriter pw = new PrintWriter(new BufferedWriter(new FileWriter( packagePath.trim() + fileName.trim() + ".java" )));
                pw.println( packageString );
                return pw;
        }
        public static void generateStruct( PrintWriter pw,
                                           String structName,
                                           Vector structTypeSpecVector,
                                           Vector structDeclaratorVector,
					   boolean isException)
        {
            /* Debug */
            if( structTypeSpecVector == null ) System.out.println( "structTypeSpecVector is null" );
            if( structDeclaratorVector == null ) System.out.println( "structDeclaratorVector is null" );
            /* Debug */
            pw.println( "final public class " + structName );
	    if( isException )
            	pw.println( "      extends Exception" );
            pw.println( "      implements org.omg.CORBA.portable.IDLEntity {" );
            pw.println( "   // instance variables" );
            int numFields = structTypeSpecVector.size();
            for( int i = 0; i < numFields; i++ )
            {
                pw.print( "    public " );
		String javaType = CodeGenX.mapVariableType( structTypeSpecVector.elementAt(i).toString() );
                pw.print( javaType + " " );
                StructDeclarator declarator = (StructDeclarator)structDeclaratorVector.elementAt(i);
                /*Debug Section */
                if( declarator == null )
                    System.out.println("Debug: declarator null in generate struct");
                /* Debug */
                pw.println( declarator.printAsDeclaration() + ";" );
            }
            pw.println( "    public " + structName + "() {}" );
            pw.print( "    public " + structName + "(" );
            StructDeclarator firstDeclarator = (StructDeclarator)structDeclaratorVector.elementAt(0);
            String firstTypeSpec = CodeGenX.mapVariableType( structTypeSpecVector.elementAt(0).toString() );
            pw.print( firstTypeSpec + " " + firstDeclarator.printAsParameter() );
            for( int i = 1; i < numFields; i++ )
            {
                StructDeclarator declarator = (StructDeclarator)structDeclaratorVector.elementAt(i);
                String typeSpec = CodeGenX.mapVariableType( structTypeSpecVector.elementAt(i).toString() );
                pw.print( ", " + typeSpec + " " + declarator.printAsParameter() );
            }
            pw.println( ")" );
            pw.println( "    {" );
            for( int i = 0; i < numFields; i++ )
            {
                StructDeclarator declarator = (StructDeclarator)structDeclaratorVector.elementAt(i);
                pw.print("        this." + declarator.printAsAssignment() + " = " );
                pw.println( declarator.printAsAssignment() + ";" );
            }
            pw.println( "    }" );
            pw.println( "}" );
            pw.flush();
        }

        public static void generateOpDclStartCode( String returnValueIDLType , String opName , String packageString , String packagePath , PrintWriter pw )
        {
                System.out.println( returnValueIDLType );
                pw.print( "\t" + mapReturnType( returnValueIDLType ) + " " + opName );
                pw.flush();
        }

        public static void printWord( String word , PrintWriter pw )
        {
                pw.print( word );
                pw.flush();
        }

        public static void generateParamCode( String type , String IDLtype , String name , PrintWriter pw )
        {
                if( type.equals( "in" ) )
                {
                        pw.print( mapVariableType( IDLtype ) );
                }else{
                        pw.print( mapVariableHolder( IDLtype ) );
                }
                pw.print( " " + name );
                pw.flush();
        }
        public static void generateUnion ( PrintWriter pw, String unionName, Vector unionTypeSpecVector, Vector unionDeclaratorVector )
        {
	    String javaType = "";
            pw.println("final public class "+unionName+" implements org.omg.CORBA.portable.IDLEntity" );
            pw.println("{" );
	    pw.println("  int choice = 0;");
            pw.println("  // instance variables");
	    int numFields = unionTypeSpecVector.size();
            for( int i = 0; i < numFields; i++ )
            {
	        String type = unionTypeSpecVector.elementAt(i).toString(); 
		javaType = CodeGenX.mapVariableType( type );
                pw.print("  public "+javaType+" ");
                UnionDeclarator declarator = (UnionDeclarator)unionDeclaratorVector.elementAt(i);
                pw.println( declarator.printAsDeclaration() + ";" );
            }
            pw.println("  public "+unionName+"() {}" );
            pw.print("  public "+unionName+"(");
            UnionDeclarator firstDeclarator = (UnionDeclarator)unionDeclaratorVector.elementAt(0);
            String firstType = unionTypeSpecVector.elementAt(0).toString();
	    javaType = CodeGenX.mapVariableType( firstType );
            pw.print( javaType + " " + firstDeclarator.printAsParameter() );
            for( int i = 1; i < numFields; i++ )
            {
                UnionDeclarator declarator = (UnionDeclarator)unionDeclaratorVector.elementAt(i);
                String type = unionTypeSpecVector.elementAt(i).toString();
		javaType = CodeGenX.mapVariableType( type );
                pw.print(", "+javaType+" "+declarator.printAsParameter() );
            }
            pw.println(")");
            pw.println("  {");
            for( int i = 0; i < numFields; i++ )
            {
                UnionDeclarator declarator = (UnionDeclarator)unionDeclaratorVector.elementAt(i);
                pw.print("  this." + declarator.printAsAssignment()+" = ");
                pw.println(declarator.printAsAssignment()+";");
            }
            pw.println("  }");
	    pw.println("  public int discriminator()");
	    pw.println("  {");
	    pw.println("    return choice;");
	    pw.println("  }");
	    pw.println();
	    for( int i = 0; i < numFields; i++ )
	    {
	        String field = unionDeclaratorVector.elementAt(i).toString();
		String type = unionTypeSpecVector.elementAt(i).toString();
		javaType = CodeGenX.mapVariableType( type );
	    pw.println("  public void "+field+"("+javaType+" _"+field+")");
	    pw.println("  {");
	    pw.println("    choice = "+i+";");
	    pw.println("    "+field+" = _"+field+";");
	    pw.println("  }");
	    pw.println();
	    }  
            pw.println("}");
            pw.close();
        
        }
/*
CORBA::UNKNOWN org.omg.CORBA.UNKNOWN
CORBA::BAD_PARAM org.omg.CORBA.BAD_PARAM
CORBA::NO_MEMORY org.omg.CORBA.NO_MEMORY
CORBA::IMP_LIMIT org.omg.CORBA.IMP_LIMIT
CORBA::COMM_FAILURE org.omg.CORBA.COMM_FAILURE
CORBA::INV_OBJREF org.omg.CORBA.INV_OBJREF
CORBA::NO_PERMISSION org.omg.CORBA.NO_PERMISSION
CORBA::INTERNAL org.omg.CORBA.INTERNAL
CORBA::MARSHAL org.omg.CORBA.MARSHAL
CORBA::INITIALIZE org.omg.CORBA.INITIALIZE
CORBA::NO_IMPLEMENT org.omg.CORBA.NO_IMPLEMENT
CORBA::BAD_TYPECODE org.omg.CORBA.BAD_TYPECODE
CORBA::BAD_OPERATION org.omg.CORBA.BAD_OPERATION
CORBA::NO_RESOURCES org.omg.CORBA.NO_RESOURCES
CORBA::NO_RESPONSE org.omg.CORBA.NO_RESPONSE
CORBA::PERSIST_STORE org.omg.CORBA.PERSIST_STORE
CORBA::BAD_INV_ORDER org.omg.CORBA.BAD_INV_ORDER
CORBA::TRANSIENT org.omg.CORBA.TRANSIENT
CORBA::FREE_MEM org.omg.CORBA.FREE_MEM
CORBA::INV_IDENT org.omg.CORBA.INV_IDENT
CORBA::INV_FLAG org.omg.CORBA.INV_FLAG
CORBA::INTF_REPOS org.omg.CORBA.INTF_REPOS
CORBA::BAD_CONTEXT org.omg.CORBA.BAD_CONTEXT
CORBA::OBJ_ADAPTER org.omg.CORBA.OBJ_ADAPTER
CORBA::DATA_CONVERSION org.omg.CORBA.DATA_CONVERSION
CORBA::OBJECT_NOT_EXIST org.omg.CORBA.OBJECT_NOT_EXIST
CORBA::TRANSACTION_REQUIRED org.omg.CORBA.TRANSACTION_REQUIRED
CORBA::TRANSACTION_ROLLEDBACK org.omg.CORBA.TRANSACTION_ROLLEDBACK
CORBA::INVALID_TRANSACTION org.omg.CORBA.INVALID_TRANSACTION
CORBA::INV_POLICY org.omg.CORBA.INV_POLICY
CORBA::CODESET_INCOMPATIBLE org.omg.CORBA.CODESET_INCOMPATIBLE
*/

}
