/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

package ZEN_IDL.be;

import java.io.*;
import java.util.*;
import ZEN_IDL.ast.*;
import ZEN_IDL.be.helperUtils.*;

/*********************************************************
* Takes care of printing to the specified file with
* the specified intendation.
*
* @author Olav Haugan
*
* 14.10.01
*********************************************************/
public class MQCPrinter
{
	private int indent;	/* The number of tabs to print */
	private PrintWriter pw; /* The outputstream to print to */
	
	/*********************************************************
	* Constructor. 
	*********************************************************/
	public MQCPrinter()
	{
	}	
	
	/*********************************************************
	* Constructor. Initializes variables.
	*********************************************************/
	public MQCPrinter(PrintWriter pw, int indent)
	{
		this.pw = pw;
		this.indent = indent;
	}	

	/*********************************************************
	* Sets the print writer.
	* 
	* @param PrintWriter The output stream
	*********************************************************/
	public void setPW(PrintWriter pw)
	{
		this.pw = pw;
	}	

	/*********************************************************
	* Returns the print writer.
	* 
	* @return PrintWriter The output stream
	*********************************************************/
	public PrintWriter getPW()
	{
		return pw;
	}	

	/*********************************************************
	* Sets the number of indents.
	* 
	* @param int The number of indents
	*********************************************************/
	public void setIndent(int indent)
	{
		this.indent = indent;
	}	

	/*********************************************************
	* Gets the number of indents.
	* 
	* @return int The number of indents
	*********************************************************/
	public int getIndent()
	{
		return indent;
	}	

	/*********************************************************
	* Prints a newline
	*********************************************************/
	public void println()
	{
		pw.println();
	}	

	/*********************************************************
	* Prints the text with the number of tabs specified
	* with endline.
	* 
	* @param String The text to be written
	*********************************************************/
	public void println(String text)
	{
		for(int i = 0; i < indent; i++)
			pw.print("\t");
		pw.println(text);
	}	

	/*********************************************************
	* Prints the text with the number of tabs specified
	* without endline.
	* 
	* @param String The text to be written
	*********************************************************/
	public void print(String text)
	{
		for(int i = 0; i < indent; i++)
			pw.print("\t");
		pw.print(text);
	}	

	/*********************************************************
	* Prints the text without tabs with endline.
	* 
	* @param String The text to be written
	*********************************************************/
	public void nprintln(String text)
	{
		pw.println(text);
	}	

	/*********************************************************
	* Prints the text without tabs without endline.
	* 
	* @param String The text to be written
	*********************************************************/
	public void nprint(String text)
	{
		pw.print(text);
	}	

	/*********************************************************
	* Flushes the stream
	*********************************************************/
	public void flush()
	{
		pw.flush();
	}	

	/*********************************************************
	* Closes the stream
	*********************************************************/
	public void close()
	{
		pw.close();
	}	
}
