/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

//package timing;

import java.io.*;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;


public class Server 
{
    public static void main(String[] args) 
    {
        if( args.length != 1 ) 
        {
            System.out.println(
                "Usage: jaco timing.Server <ior_file>");
            System.exit( 1 );
        }

        try 
        {            
            //init ORB
            ORB orb = ORB.init( args, null );

            //init POA
            POA poa = 
                POAHelper.narrow( orb.resolve_initial_references( "RootPOA" ));

            poa.the_POAManager().activate();

            // create a Foo object
            timing.FooImpl fooImpl = new timing.FooImpl();        
    
            // create the object reference
            org.omg.CORBA.Object obj = 
                poa.servant_to_reference( fooImpl );

            PrintWriter pw = 
                new PrintWriter( new FileWriter( args[ 0 ] ));

            // print stringified object reference to file
            pw.println( orb.object_to_string( obj ));
            
            pw.flush();
            pw.close();
    
            ShowMemory();

            // wait for requests
            orb.run();

            ShowMemory();
        }
        catch( Exception e ) 
        {
            System.out.println( e );
        }
    }

  //-- Memory usage for Java --
  static void ShowMemory() {
    long tMem, fMem, uMem, fMem1, uMemMax, fMemMax;
    Runtime runtime = Runtime.getRuntime();

    tMem    = runtime.totalMemory();
    fMem    = runtime.freeMemory();
    fMemMax = fMem;
    if (tMem == fMem) {
      // TINI hack, tMem and fMem are reported as equal
      tMem = 334240;  // MAGIC NUMBER appears to be the proper value
    }
    uMem    = tMem - fMem;
    System.out.println("c.Memory (t/f/u): \t"+tMem+" \t"+fMem+" \t"+uMem);

    System.gc();
    try {
      Thread.sleep(500);
    } catch (InterruptedException ie) {}

    for (int i = 0;  i < 100;  i++) {
      try {
        Thread.sleep(500);
      } catch (InterruptedException ie) {}

      fMem1   = runtime.freeMemory();

      ;

      if (fMem1 > fMemMax ) {
        fMemMax = fMem1;
      } else {
        break;
      }
    }
    uMemMax = tMem - fMemMax;

    System.out.println("c.Memory (t/f/u): \t"+tMem+" \t"+fMemMax+" \t"+uMemMax);
  }
}
