// client.cpp,v 1.5 2002/01/29 20:21:07 okellogg Exp

#include "TimingC.h"
#include "ace/Get_Opt.h"
#include <time.h>

ACE_RCSID(foo, client, "client.cpp,v 1.5 2002/01/29 20:21:07 okellogg Exp")

const char *ior = "file://timing.ior";
long cnt = 1000;  // message call count value

int
parse_args (int argc, char *argv[])
{
  ACE_Get_Opt get_opts (argc, argv, "c:k:");
  int c;

  while ((c = get_opts ()) != -1)
    switch (c)
      {
      case 'c':
        cnt = atoi( get_opts.opt_arg ());
        break;

      case 'k':
        ior = get_opts.opt_arg ();
        break;

      case '?':
      default:
        printf("%s\n", get_opts.opt_arg());
        ACE_ERROR_RETURN ((LM_ERROR,
                           "usage:  %s "
                           "-c <count> "
                           "-k <ior> "
                           "\n",
                           argv [0]),
                          -1);
      }
  // Indicates sucessful parsing of the command line
  return 0;
}

int
main (int argc, char *argv[])
{
  ACE_TRY_NEW_ENV
    {
      CORBA::ORB_var orb =
        CORBA::ORB_init (argc, argv, "" ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;

      if (parse_args (argc, argv) != 0)
        return 1;

      CORBA::Object_var tmp =
        orb->string_to_object(ior ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;

      timing::foo_var timing =
        timing::foo::_narrow(tmp.in () ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;

      if (CORBA::is_nil (timing.in ()))
        {
          ACE_ERROR_RETURN ((LM_DEBUG,
                             "Nil timing::foo reference <%s>\n",
                             ior),
                            1);
        }

      printf("Cnt=%d, Starting the timing loop...\n", cnt);
      timeval start, stop;
      gettimeofday(&start, 0);
      CORBA::Long in_long = 1;
      for (int i = 0;  i < cnt;  i++) 
        {
          CORBA::Long the_long =
            timing->bar (i ACE_ENV_SINGLE_ARG_PARAMETER);
          ACE_TRY_CHECK;

//          ACE_DEBUG ((LM_DEBUG, "(%P|%t) - long returned <%d>\n", the_long));
        }
      gettimeofday(&stop, 0);
      printf("start: %d.%6.6d  stop: %d.%6.6d\n",
              start.tv_sec, start.tv_usec, stop.tv_sec, stop.tv_usec);        
      double elapsed = ( stop.tv_sec * 1000000. + stop.tv_usec -
                        (start.tv_sec * 1000000. + start.tv_usec) ) / 1000000.;
      printf("elapsed: %f\n", elapsed);
      printf("*** Total time (s): %f  *** Average time (us):\t%.3f\n",
              elapsed, (1000000. * elapsed)/cnt);

      orb->destroy (ACE_ENV_SINGLE_ARG_PARAMETER);
      ACE_TRY_CHECK;
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION,
                           "Exception caught:");
      return 1;
    }
  ACE_ENDTRY;

  return 0;
}
