/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

package mqc.exception;

/**
 * Base class for system exceptions
 * 
 * @author Olav Haugan
 *
 * 14.10.01
 */
public class SystemException extends java.lang.RuntimeException
{
   public static final int COMPLETED_YES = 0;
   public static final int COMPLETED_NO = 1;
   public static final int COMPLETED_MAYBE = 2;

   public int minor;
   public int completionStatus;

   /**
    * Constructor. Initializes variables.
    *
    * @param reason  exception reason
    * @param minor  minor value
    * @param completionStatus  completion status
    */
   public SystemException(String reason, int minor, int completionStatus)
   {
      super(reason);
      this.minor = minor;
      this.completionStatus = completionStatus;
   }

   /**
    * Converts the exception to a string
    */
   public String toString()
   {
      StringBuffer ex = new StringBuffer();
      ex.append("SystemException\n");
      ex.append("Reason: " + getMessage() + "\n");
      ex.append("Minor: " + minor + "\n");
      ex.append("Completed: ");
      switch(completionStatus)
      {
      case COMPLETED_YES:
         ex.append("Yes\n");
         break;
      case COMPLETED_NO:
         ex.append("No\n");
         break;
      case COMPLETED_MAYBE:
         ex.append("Maybe\n");
         break;
      default:
         ex.append("Unknown status\n");
      }
      return ex.toString();
   }
}
