/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

package mqc.protocols;

/**
 * Reply holder interface 
 * 
 * @author Eivind Naess
 * @author Kim Christian Swenson
 */
public interface Reply extends Marshalable
{

   /* Possible reply status values */
   public static final int NO_EXCEPTION          = 0;
   public static final int USER_EXCEPTION        = 1;
   public static final int SYSTEM_EXCEPTION      = 2;

   /**
    * Return the Reply Status field of the demarshalled reply messages
    * @return Reply the deMarshalled message's reply status
    */
   public int getReplyStatus();

   public void setReplyStatus( int status );

   /**
    * Returns the Completion Status field
    * @return The Completion Status
    */
   public int getCompletionStatus();

   public void setCompletionStatus( int status );

   /**
    * Returns the Exception Minor filed
    * @return An integer representing the exception Minor
    */
   public int getExceptionMinorCode();

   public void setExceptionMinorCode( int minor );


   public int getExceptionMinorVMCID();

   public void setExceptionMinorVMCID( int vmcid );

   /**
    * Returns the Exception ID thrown
    * @return String representation of the Exception ID
    */
   public String getExceptionID();

   public void setExceptionID( String id );

   /** retrives the request ID */
   public int getRequestID();

   /**
    * Sets the message's request ID
    */
   public void setRequestID( int reqID );

   public Body getBody();

   public void setBody( Body body );

   public boolean isBigEndian();

   public void setBigEndian( boolean bigEndian );

   public int getHdrSize();
}
