/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

include(`Config_macros.m4')

package mqc.transports;

import mqc.*;
import mqc.protocols.*;
import mqc.holders.*;
import java.io.*;
import java.util.*;
__CLDC__(
import javax.microedition.io.*;
)
__J2SE__(
import java.net.*;
)

/**
 * Base class for all the UDP session handlers.
 * 
 * @author Thor Egil Skaug    initial
 * @version 1.0
 */

public abstract class S_UDPGenericHandler
{
   protected S_ORB m_orb = null;
   protected byte[] m_recvBuf = null;
   protected byte[] m_sendBuf = null;
   protected int m_nMessageSize=0;
   protected int m_nHeaderSize=0;
   __J2SE__(
   protected String m_strClientName;
   protected int m_nClientPort;
   protected DatagramSocket m_clientSocket = null;
   protected DatagramPacket m_sendPacket = null;
   protected DatagramPacket m_recvPacket = null;   
   )
   __CLDC__(
   protected DatagramConnection m_clientSocket = null;
   protected Datagram m_sendPacket = null;
   protected Datagram m_recvPacket = null;
   protected String m_strClientSessionURL;
   )
   __TEMPSPACE__(
   protected Vector m_vMessageBuffer = null;
   )
   __SECURITY__(
   protected int m_nPadOffset=0;                                             //offset of encryption alignment padding
   )
   __SECURITY_MAC__(
   protected int m_nMacOffset=0;                                             //offset of mac
   )
   __SECURITY_MD__(
   __SECURITY_MD_NOT_HMAC__(
   protected int m_nMdOffset=0;                                              //offset of md
   )
   )
   
   __J2SE__(
   public S_UDPGenericHandler(DatagramSocket clientSocket,String strClientName,int nClientPort,S_ORB orb)
   )
   __CLDC__(
   public S_UDPGenericHandler(DatagramConnection clientSocket,String strClientURL, S_ORB orb)
   )
   {
      m_clientSocket = clientSocket;
      m_orb = orb;
      __J2SE__(
      m_strClientName = strClientName;
      m_nClientPort = nClientPort;
      )
      __CLDC__(
      m_strClientSessionURL = strClientURL;
      )
      __TEMPSPACE__(
      m_vMessageBuffer = new Vector();
      )
     
      __FIX_SIZE__(
      //in fixed size messages, the offsets are always the same
      m_nMessageSize = Config.intPacketSize;
      __SECURITY_MD__(
      __SECURITY_MD_NOT_HMAC__(
      m_nMdOffset = m_nMessageSize;                                        //do not add MD len if the MD is for
      m_nMessageSize += ORB.msgDigestLen;                                  //an HMAC (ie, no double counting)
      )
      )
      __SECURITY_MAC__(
      m_nMacOffset = m_nMessageSize;
      m_nMessageSize += ORB.msgMacLen;
      )
      __SECURITY__(
      m_nPadOffset = m_nMessageSize;                                       //sets the padding offset (no double padding!!)
      m_nMessageSize = m_orb.cipherDecrypt.getOutputSize(m_nMessageSize);  //adds whatever padding is needed to the message size
      )
      m_recvBuf = new byte [m_nMessageSize];                               //create the fixed size receive buffer
      m_sendBuf = new byte [m_nMessageSize];                            //create the fixed size send buffer
      )
      
      __VAR_SIZE__(
      //for UDP we cannot have a growing receive buffer, so
      //if Config.intPacketSize is set to zero, we set this static
      m_recvBuf = new byte[__MAX_UDP_DATAGRAM_SIZE__];                     //tes: TODO change this to a macro constant
      )   
   }
}
