/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

include(`Config_macros.m4')

package mqc.transports;

import mqc.*;
import mqc.ior.*;
import mqc.holders.*;
import mqc.transports.*;
import mqc.protocols.*;
import mqc.util.*;
import java.io.*;
import java.util.*;
__J2SE__(import java.net.*;)
__CLDC__(import javax.microedition.io.*;)
__SECURITY__(import mqc.security.GeneralSecurityException;)
__SECURITY_MD__(import mqc.security.DigestException;)
__SECURITY_MAC__(import mqc.security.ShortBufferException;)

/**
 * UDPTransport handles unreliable udp transport.
 *
 * @author Thor Egil Skaug    security and optimization
 * @author Kevin Dorow        groupcomm
 * @author Eivind Naess       cldc
 * @author Tarana Damania     initial
 * @author Olav Haugan        initial
 * @version 1.00
 */

public class UDPTransport extends UDPGenericTransport
{
   __J2SE__(
   private DatagramPacket m_sendPacket = null;
   private DatagramPacket m_recvPacket = null;
   )
   __CLDC__(
   private Datagram m_sendPacket = null;
   private Datagram m_recvPacket = null;
   )

   __TEMPSPACE__(
   int lastRequestID = -1;
   )

   public UDPTransport(mqc.Delegate delegate)
   {
      super(delegate);

      m_in = new CDRInputStream();
      m_out = new CDROutputStream(m_recvBuf.length, 0, false);

      init();
   }

   public boolean init()
   {
      try
      {
         //connect to the server
         establishServerConnection(Config.UDP);
      }
      __J2SE__(
      catch(SocketException se)
      {
         se.printStackTrace();
         return false;
      }
      )      
      catch(IOException ioe)
      {
         ioe.printStackTrace();  
         return false;
      }
      return true;
   }

   public boolean send(byte[] buffer, int nLen)
   {
      __VAR_SIZE__(
      m_nHeaderSize = Message.getHdrSize(buffer);
      m_nMessageSize = nLen;    
      __SECURITY_MD__(
      __SECURITY_MD_NOT_HMAC__(
      m_nMdOffset = m_nMessageSize;
      m_nMessageSize += ORB.msgDigestLen;
      )
      )

      __SECURITY_MAC__(
      m_nMacOffset = m_nMessageSize;
      m_nMessageSize += ORB.msgMacLen;
      )

      __SECURITY__(
      m_nPadOffset = m_nMessageSize;
      m_nMessageSize += ORB.cipherDecrypt.getOutputSize(m_nMessageSize - m_nHeaderSize) + m_nHeaderSize;
      )

      if(m_sendBuf == null || m_sendBuf.length < m_nMessageSize)
      {
         //create or grow the sendbuffer
         m_sendBuf = new byte[m_nMessageSize];   
      }
      )

      try
      {
         __SECURITY_MD__(
         __SECURITY_MD_NOT_HMAC__(
         try
         {
            ORB.msgDigest.reset();
            ORB.msgDigest.update(buffer, 0, m_nMdOffset);
            ORB.msgDigest.digest(buffer, m_nMdOffset, ORB.msgDigestLen);
         }
         catch(DigestException e)
         {
            e.printStackTrace();
         }
         )
         )
         __SECURITY_MAC__(
         try
         {
            ORB.msgMac.reset();
            ORB.msgMac.update(buffer, 0, m_nMacOffset);
            ORB.msgMac.doFinal(buffer, m_nMacOffset);
         }
         catch(ShortBufferException sbe)
         {
            sbe.printStackTrace();
         }
         catch(IllegalStateException ise)
         {
            ise.printStackTrace();
         }
         )

         __SECURITY__(
         try
         {
            __FIX_SIZE__(
            //in fixedsize we encrypt the entire buffer
            ORB.cipherEncrypt.doFinal(buffer, 0, m_nPadOffset, m_sendBuf, 0);
            )
            __VAR_SIZE__(
            //in varsize we leave the header unencrypted
            System.arraycopy(buffer, 0, m_sendBuf, 0, m_nHeaderSize);
            __DEBUG__(
            System.out.println("unencrypted: ");
            //mqc.util.msgPrinter.printHex(buffer,0,m_nPadOffset);
            )
            ORB.cipherEncrypt.doFinal(buffer, m_nHeaderSize, m_nPadOffset - m_nHeaderSize, m_sendBuf, m_nHeaderSize);
            __DEBUG__(
            System.out.println("encrypted: ");
            //mqc.util.msgPrinter.printHex(m_sendBuf,0,m_nPadOffset);
            )
            )
         }
         catch(GeneralSecurityException gse)
         {
            gse.printStackTrace();         
         }
         buffer = m_sendBuf;
         )   
      }
      catch(Exception e)
      {
         e.printStackTrace();
         return false;
      }
      try
      {
         if(m_sendPacket == null)
         {
            __J2SE__(
            m_sendPacket = new DatagramPacket(buffer, m_nMessageSize, InetAddress.getByName(m_strServerName), m_nServerSessionPort);   
            )
            __CLDC__(
            m_sendPacket = m_serverSocket.newDatagram(buffer, m_nMessageSize, m_strServerSessionURL);
            )
         }
         else
         {
            __J2SE__(
            m_sendPacket.setData(buffer);
            )
            __CLDC__(
            m_sendPacket.setData(buffer, 0, m_nMessageSize);   
            )
         }
         __TEMPORAL__(
         for(int i=0;i<Config.temporal;i++)
         {
         )
         m_serverSocket.send(m_sendPacket);
         __TEMPORAL__(
         }
         )
      }
      catch(IOException ioe)
      {
         ioe.printStackTrace();
         return false;
      }
      return true;          
   }

   public boolean waitForReply(mqc.protocols.Reply msg)
   {
      __J2SE__(
      if(m_recvPacket == null)
      {
         m_recvPacket = new DatagramPacket(m_recvBuf, m_recvBuf.length);   
      }
      else
      {
         m_recvPacket.setData(m_recvBuf);
      }
      )
      __CLDC__(
      if(m_recvPacket == null)
      {
         try
         {
            m_recvPacket = m_serverSocket.newDatagram(m_recvBuf, m_recvBuf.length);
         }
         catch(IOException ioe)
         {
            ioe.printStackTrace();   
         }
      }
      )
      //receive the packet
      try
      {
         m_serverSocket.receive(m_recvPacket);
      }
      catch(IOException ioe)
      {
         ioe.printStackTrace();
         return false;
      }
      int nLen = m_recvPacket.getLength();

      __VAR_SIZE__(
      boolean bigEndian = Message.isBigEndian( m_recvBuf );
      m_nHeaderSize = Message.getHdrSize(m_recvBuf);
      m_nMessageSize = Message.getMsgSize(m_recvBuf, bigEndian) + m_nHeaderSize;

      __SECURITY_MD__(
      __SECURITY_MD_NOT_HMAC__(
      m_nMdOffset = m_nMessageSize;
      m_nMessageSize += ORB.msgDigestLen;
      )
      )
      __SECURITY_MAC__(
      m_nMacOffset = m_nMessageSize;
      m_nMessageSize += ORB.msgMacLen;
      )
      __SECURITY__(
      m_nPadOffset = m_nMessageSize;
      m_nMessageSize = ORB.cipherDecrypt.getOutputSize(m_nMessageSize-m_nHeaderSize) + m_nHeaderSize;

      //variable size decrypt
      try
      {
         ORB.cipherDecrypt.doFinal(m_recvBuf, m_nHeaderSize, m_nPadOffset - m_nHeaderSize, m_recvBuf, m_nHeaderSize);
      }
      catch(GeneralSecurityException gse)
      {
         gse.printStackTrace();   
      }      
      )
      )

      __FIX_SIZE__(
      //in FIX_SIZE all the offsets are constant throughout the lifetime of this object
      __SECURITY__(
      try
      {
         ORB.cipherDecrypt.doFinal(m_recvBuf,0,nLen,m_recvBuf,0);
      }
      catch(GeneralSecurityException gse)
      {
         gse.printStackTrace();   
      }
      )
      )

      __SECURITY_MAC__(
      ORB.msgMac.reset();
      ORB.msgMac.update(m_recvBuf, 0, m_nMacOffset);

      byte[] compMAC = ORB.msgMac.doFinal();
      byte[] recvMAC = new byte[ORB.msgMacLen];

      System.arraycopy(m_recvBuf, m_nMacOffset, recvMAC, 0, ORB.msgMacLen);

      if (!ORB.msgMac.isEqual(compMAC, recvMAC))
      {
         System.err.println("***Invalid MAC");
         return false;
      }
      )

      __SECURITY_MD__(
      __SECURITY_MD_NOT_HMAC__(
      ORB.msgDigest.reset();
      ORB.msgDigest.update(m_recvBuf, 0, m_nMdOffset);

      byte[] compMD = ORB.msgDigest.digest();
      byte[] recvMD = new byte[ORB.msgDigestLen];

      System.arraycopy(m_recvBuf, m_nMdOffset, recvMD, 0, ORB.msgDigestLen);

      if (!ORB.msgDigest.isEqual(compMD, recvMD))
      {
         System.err.println("***Invalid message digest");
         return false;
      }
      )
      )

      // Tell CDRInputStream to use recvbuf
      m_in.setBuffer( m_recvBuf );

      // DeMarshal message into reply
      msg.deMarshal( m_in );

      __TEMPORAL__(
      int tmpID = msg.getRequestID();
      if ( tmpID > lastRequestID )
      {
         lastRequestID = tmpID;
      }
      else
      {
         // Duplicate
         __DEBUG5__(System.err.println( "UDPTransport.waitForReply(): skipped duplicate with ID " + tmpID );)
         return false;
      }
      )

      return true;
   }

   __TEMPORAL__(
   public int skipReply( Reply rep )
   {
      return 0;
   }   
   )
}
